//-----------------------------------Halsafar SLAQ Library-------------------------------//
//Title:            Halsafars SLAQ Library - Stack, List, Array, Queue                   // 
//File:             cls_SLAQ.h                                                           //
//Functionality:    SLAQ is an abstract class not intended for initilization             //
//Date:             March 3, 2005 - 18:38                                                //
//Author:           Steve Damm (Halsafar)                                                //
//                                                                                       //
//Notes:            Use of this array is entitled to those who notify me prior to        //
//                  implimentation into their source.  Credit to Steve Damm must be      //
//                  included once within the users source.                               //
//---------------------------------------------------------------------------------------//

#pragma once
#ifndef __SLAQ
#define __SLAQ

//The only two necessary includes
#include <malloc.h>
#include <memory.h>


//Some helper macro's
#define SAFE_FREE(PTR)      if(PTR != 0) { free(PTR);     PTR=0;}
#define SAFE_DELETE(PTR)    if(PTR != 0) { delete PTR;    PTR=0;}
#define SAFE_DELETEARY(PTR) if(PTR != 0) { delete [] PTR; PTR=0;}


//Typedefs to use modern notation
typedef unsigned long DWORD;
typedef unsigned int  UINT;

//A set of error flags that are filled when non-terminating exceptions come up
#define ERR_NOSPACE 0x0001

//The SLAQ class
template <class _TY> class SLAQ;      //Prototype
template <class _TY>
class SLAQ 
{
protected:
    //--------------------
    //Protected members
    _TY         *m_Begin;
    _TY         *m_End;

    size_t       m_SizeOf;
    size_t       m_TotalSize;

    DWORD        m_dwMaxIndex;
    DWORD        m_dwErrorFlag;
    //--------------------
    
    
    //--------------------
    //Virtual Protected members
    virtual void Destroy() = 0;
    //--------------------

    //--------------------
    //Private Functions
    virtual bool Allocate(UINT uiSize) {        //Size is passed as number of index's requested
        SAFE_FREE(m_Begin);

        m_dwMaxIndex = uiSize;
        m_TotalSize  = uiSize * m_SizeOf;
        m_Begin       = (_TY*)malloc(m_TotalSize);
        if (!m_Begin && uiSize != 0) {
            m_dwErrorFlag = ERR_NOSPACE;
            return false;
        }
        m_End    = m_Begin + uiSize;

        memset(m_Begin, 0, m_TotalSize);

        return true;
    }

    virtual bool ReAllocate(UINT uiSize) {      //Size is passed as new total number of index's
        m_TotalSize = uiSize * m_SizeOf;
        
        m_Begin = (_TY*)realloc(m_Begin, m_TotalSize);
        if (!m_Begin && uiSize != 0) {
            m_dwErrorFlag = ERR_NOSPACE;
            return false;
        }
        m_End = m_Begin + uiSize;
        
        if (uiSize > m_dwMaxIndex) {    //If we are enlarging the array then we must set to 0
            memset(m_Begin+m_dwMaxIndex, 0, m_SizeOf * (uiSize-m_dwMaxIndex));
        }

        //We want to LESS THAN for any dwMaxIndex comparison
        m_dwMaxIndex = uiSize;
        
        return true;
    }
    //--------------------
public:
    //--------------------
    //Constructors
    SLAQ() : 
      m_Begin(0), m_End(0), m_SizeOf(0), m_TotalSize(0),
          m_dwMaxIndex(0), m_dwErrorFlag(0) {
      }

    virtual ~SLAQ() {
    }
    //--------------------

    //--------------------
    //SLAQ Managing functions
    _TY* Push(_TY _ITEM) {                      //Push _TYPE onto the top of the array
        if (!ReAllocate(m_dwMaxIndex + 1)) {
            return 0;
        }
        
        *End() = _ITEM;

        //cout << "SLAQ Push: " << _ITEM << endl;
        return End();
    }

    _TY Pop() {
        if (m_dwMaxIndex == 0) {
            return 0;
        }

        _TY _ITEM;
        _ITEM = *End();
        if (!ReAllocate(m_dwMaxIndex - 1)) {    //Pop the top member off the array, resize.
            return 0;
        }

        
        //cout << "SLAQ PoP: " << _ITEM << endl;
        return _ITEM;
    }

    _TY &Peek() {
        return *(End());
    }
    //--------------------

    //--------------------
    //Helper functions
    size_t SizeOf()       const{            //SizeOf
        return sizeof(_TY);
    }

    size_t SizeOfArray()  const{            //SizeOf Array
        return m_TotalSize;
    }

    _TY*     Begin()        const{          //Returns the BEGIN PTR
        return m_Begin;
    }

    _TY*     End()          const{          //Returns the END PTR
        return m_End-1;
    }

    DWORD  Upperbound()   const{            //Returns the UPPERBOUND
        return m_dwMaxIndex;
    }

    void   Clear()             {            //Destroys the array
        Destroy();
    }

    DWORD  Bad()          const{            //Checks for a bad bit
        return m_dwErrorFlag;
    }
    //--------------------

    //--------------------
    //Overloaded operators
    _TY *operator +(int i) {
        return (m_Begin + i);
    }

    _TY *operator -(int i) {
        return (m_Begin - i);
    }

    
    _TY &operator *() {
        return *m_Begin;
    }
    //--------------------
};

#endif