//*********************************
//Halsafar's 3D City - Query Performance Counter - HEADER
//
//Functions: Runs a QueryPerformance Timers with Ez functionality
//
//Date: August 2004, Modified February 2005
//
//Author: Steve Damm
//
//*********************************
//Class Guard
#pragma once
#ifndef __TIMER
#define __TIMER

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <string>
using namespace std;

//The Timer Engine
class TimerEngine
{

private:
	
	LARGE_INTEGER	Start;			//StartTime
	LARGE_INTEGER	Finish;			//FinishTime
	LARGE_INTEGER	Frequency;		//Counter Frequency
	float			Delta;			//Time passed

	
	int			Hours;		 //Hours past
	int			Minutes;	 //Minutes past
	int			Seconds;	 //Seconds past
	float		MilliSeconds;//Milliseconds past

	string		strTime;	 //Holds a formatted version of Hours/Minutes/Seconds

	void SetMilliSeconds();		//Set the milliseconds
	void SetSeconds();			//Set the seconds
	void SetMinutes();			//Set the minutes
	void SetHours();			//Set the hours
	void SetTimerString();		//Set the timer string


public:
	TimerEngine();
	~TimerEngine();

	void Init()			{QueryPerformanceFrequency(&Frequency);}			//Init performance counter

	void SetStart()		{QueryPerformanceCounter((LARGE_INTEGER*)&Start);}	//Get start time
	void SetFinish()	{QueryPerformanceCounter((LARGE_INTEGER*)&Finish);}	//Set the finish time
	void SetDelta()		{Delta = (Finish.QuadPart - Start.QuadPart) / (float)Frequency.QuadPart * 1000;}	//Set Delta

	float GetDelta() 		  const {return Delta;}		//Get the Delta
	float* GetDeltaPointer()	    {return &Delta;}	//Get a pointer to the delta variable


	inline void Update() {	//Update the real-world-timer
		SetMilliSeconds();
		SetSeconds();
		SetMinutes();
		SetHours();
	}

	inline string TimerEngine::GetTimeString() {	//Get the formatted timer string
		SetTimerString();
		return strTime;
	}
};





#endif